/*-----------------------------------------------------------------------------*
 *  Copyright (c) 2013 Benjamin Cronin                                         *
 *  Licensed under the MIT license (http://opensource.org/licenses/MIT)        *
 *-----------------------------------------------------------------------------*/

var fs      = require("fs"),
    path    = require("path"),
    marked  = require("marked"),
    moment  = require("moment"),
    _       = require("underscore");
 
//=============================================================================//
// Exports
//=============================================================================//

module.exports = function(opts)
{
    /* 
        Quick, dirty, get-it-working initial implementation 
        
        Synchronous, minimal options, no error checking, no file watching, etc.
    */

    
    if (typeof opts === "string")
        opts = { base : opts };
        
    opts = _.extend({
        base    : "./wiki"
    }, opts);
        
    
    var wiki = 
    {
        entries : []
    };
    
    var files = fs.readdirSync(opts.base);
    _.each(files, function (f) 
    {
        var raw = fs.readFileSync( path.join(opts.base, f), "utf8" );
        var entry = {};
        
        // Default to the whole text...this may be updated as the JSON chunk is
        // removed
        entry.content = raw;
        
        // This indexOf() approach is rather fragile - but we're assuming this is
        // being used in simple scenarios where robustness is not the priority.
        //
        var start = raw.indexOf("{{{");
        var end = raw.indexOf("}}}", start + 3);
        if (start >= 0 && end >= 0)
        {
            var jsonChunk = raw.substr(start + 2, end - 1);
            try {
                var f = new Function("return " + jsonChunk + ";");
                var json = f();
                
                // Merge the extracted JSON and the substring that's *not* the JSON
                // into a single object.
                //
                entry = _.extend({}, json, { content : raw.substr(end + 3) });
            } 
            catch (e) 
            {
            }
        }
        entry.content = marked(entry.content);
        
        wiki.entries.push(entry);
    });
    
    // Sort by date
    wiki.entries = _.sortBy(wiki.entries, function (e) {
        return -moment(e.date, "YYYY.MM.DD").unix();
    });
    
    return function (req, res, next)
    {
        // Merge *all* the wiki into the middleware locals.  This is a design choice to
        // keep the module very simple -- and also what confines it to small, simple
        // sites.
        //
        res.locals.wiki = wiki;
        
        next();
    };
};
